@@ -66,6 +66,7 @@ function XpraWindow(client, canvas_state, wid, x, y, w, h, metadata, override_re
 	this.windowtype = null;
 	this.fullscreen = false;
 	this.saved_geometry = null;
+	this.minimized = false;
 	this.maximized = false;
 	this.focused = false;
 	this.decorations = true;
@@ -499,6 +500,10 @@ XpraWindow.prototype.set_metadata_safe = function(metadata) {
 		}
 		jQuery(this.div).css('opacity', ''+opacity);
 	}
+	if ("iconic" in metadata) {
+		this.set_minimized(metadata["iconic"]==1);
+	}
+	
 	//if the attribute is set, add the corresponding css class:
 	const attrs = ["modal", "above", "below"];
 	for (let i = 0; i < attrs.length; i++) {
@@ -669,7 +674,16 @@ XpraWindow.prototype.toggle_maximized = function() {
  * Minimizes / unminimizes the window.
  */
 XpraWindow.prototype.set_minimized = function(minimized) {
-	jQuery(this.div).toggle(200);
+	if (this.minimized==minimized) {
+		return;
+	}
+	this.minimized = minimized;
+	if (minimized) {
+		jQuery(this.div).hide(200);
+	}
+	else {
+		jQuery(this.div).show(200);
+	}
 };
 
 
@@ -677,6 +691,13 @@ XpraWindow.prototype.set_minimized = function(minimized) {
  * Toggle minimized state
  */
 XpraWindow.prototype.toggle_minimized = function() {
+	if (!this.minimized) {
+		this.client.send(["unmap-window", this.wid, True]);
+	}
+	else {
+		const geom = this.get_internal_geometry();
+		this.client.send(["map-window", this.wid, geom.x, geom.y, geom.w, geom.h, this.client._get_client_properties(this)]);
+	}
 	this.set_minimized(!this.minimized);
 };
 
@@ -1331,15 +1352,23 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 				img_data = inflated.slice(0, uncompressedSize);
 			}
 			// set the imagedata rgb32 method
-			if(img_data.length > img.data.length) {
-				paint_error("data size mismatch: wanted "+img.data.length+", got "+img_data.length+", stride="+rowstride);
+			let target_stride = width*4;
+			this.debug("draw", "got ", img_data.length, "to paint with stride", rowstride, ", target stride", target_stride);
+			if (rowstride>target_stride) {
+				//we have to set line by line to honour the rowstride
+				for (let i = 0; i < height; i++) {
+					//we want to slice one line from the img_data buffer
+					//(but without using slice or subarray because the resulting array is too big!?)
+					let line = new Uint8Array(img_data.buffer, i*rowstride, target_stride);
+					img.data.set(line, i*target_stride);
+				}
 			}
 			else {
-				this.debug("draw", "got ", img_data.length, "to paint with stride", rowstride);
 				img.data.set(img_data);
-				this.offscreen_canvas_ctx.putImageData(img, x, y);
-				painted();
 			}
+			this.offscreen_canvas_ctx.clearRect(x, y, width, height);
+			this.offscreen_canvas_ctx.putImageData(img, x, y);
+			painted();
 			this.may_paint_now();
 		}
 		else if (coding=="jpeg" || coding=="png" || coding=="webp") {
@@ -1350,6 +1379,7 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 					paint_error("invalid image size: "+j.width+"x"+j.height);
 				}
 				else {
+					me.offscreen_canvas_ctx.clearRect(x, y, j.width, j.height);
 					me.offscreen_canvas_ctx.drawImage(j, x, y);
 					painted();
 				}
