@@ -9,7 +9,7 @@ import os
 import re
 
 from xpra.client.client_widget_base import ClientWidgetBase
-from xpra.os_util import bytestostr, PYTHON2, PYTHON3, OSX, WIN32, is_Wayland
+from xpra.os_util import bytestostr, OSX, WIN32, is_Wayland
 from xpra.util import typedict, envbool, envint, WORKSPACE_UNSET, WORKSPACE_NAMES
 from xpra.log import Logger
 
@@ -47,7 +47,7 @@ class ClientWindowBase(ClientWidgetBase):
              wx, wy, ww, wh, bw, bh,
              metadata, override_redirect, client_properties,
              border, max_window_size, default_cursor_data, pixel_depth))
-        ClientWidgetBase.__init__(self, client, watcher_pid, wid, metadata.boolget("has-alpha"))
+        super().__init__(client, watcher_pid, wid, metadata.boolget("has-alpha"))
         self._override_redirect = override_redirect
         self.group_leader = group_leader
         self._pos = (wx, wy)
@@ -56,6 +56,7 @@ class ClientWindowBase(ClientWidgetBase):
         self._set_initial_position = metadata.boolget("set-initial-position", False)
         self.size_constraints = typedict()
         self.geometry_hints = {}
+        self.content_type = ""
         self._fullscreen = None
         self._maximized = False
         self._above = False
@@ -90,15 +91,15 @@ class ClientWindowBase(ClientWidgetBase):
         # used for only sending focus events *after* the window is mapped:
         self._been_mapped = False
         self._override_redirect_windows = []
-        workspace = typedict(self._client_properties).intget("workspace", None)
-        if workspace is not None:
-            workspacelog("workspace from client properties: %s", workspace)
-            #client properties override application specified workspace value on init only:
-            metadata["workspace"] = int(workspace)
-        self._window_workspace = WORKSPACE_UNSET        #will get set in set_metadata if present
-        self._desktop_workspace = self.get_desktop_workspace()  #pylint: disable=assignment-from-none
         def wn(w):
             return WORKSPACE_NAMES.get(w, w)
+        workspace = typedict(self._client_properties).intget("workspace", None)
+        workspacelog("init_window(..) workspace from client properties %s: %s", self._client_properties, wn(workspace))
+        if workspace is not None:
+            #client properties override application specified workspace value on init only:
+            metadata[b"workspace"] = workspace
+        self._window_workspace = WORKSPACE_UNSET        #will get set in set_metadata if present
+        self._desktop_workspace = self.get_desktop_workspace()  #pylint: disable=assignment-from-none
         workspacelog("init_window(..) workspace=%s, current workspace=%s",
                      wn(self._window_workspace), wn(self._desktop_workspace))
         if self.max_window_size and b"size-constraints" not in metadata:
@@ -172,9 +173,6 @@ class ClientWindowBase(ClientWidgetBase):
     def update_icon(self, img):
         raise NotImplementedError("override me!")
 
-    def is_realized(self):
-        raise NotImplementedError("override me!")
-
     def apply_transient_for(self, wid):
         raise NotImplementedError("override me!")
 
@@ -220,7 +218,7 @@ class ClientWindowBase(ClientWidgetBase):
         #WARNING: "class-instance" needs to go first because others may realize the window
         #(and GTK doesn't set the "class-instance" once the window is realized)
         if b"class-instance" in metadata:
-            self.set_class_instance(*self._metadata.strlistget("class-instance", ("xpra", "Xpra"), 2, 2))
+            self.set_class_instance(*self._metadata.strtupleget("class-instance", ("xpra", "Xpra"), 2, 2))
             self.reset_icon()
 
         if b"title" in metadata:
@@ -306,7 +304,7 @@ class ClientWindowBase(ClientWidgetBase):
 
         #apply window-type hint if window has not been mapped yet:
         if b"window-type" in metadata and not self.is_mapped():
-            window_types = metadata.strlistget("window-type")
+            window_types = metadata.strtupleget("window-type")
             self.set_window_type(window_types)
 
         if b"role" in metadata:
@@ -322,7 +320,7 @@ class ClientWindowBase(ClientWidgetBase):
             if opacity<0:
                 opacity = 1
             else:
-                opacity = min(1, opacity/float(0xffffffff))
+                opacity = min(1, opacity/0xffffffff)
             #requires gtk>=2.12!
             if hasattr(self, "set_opacity"):
                 self.set_opacity(opacity)
@@ -331,7 +329,7 @@ class ClientWindowBase(ClientWidgetBase):
             new_alpha = metadata.boolget("has-alpha")
             if new_alpha!=self._has_alpha:
                 l = alphalog
-                if PYTHON3 and not WIN32:
+                if not WIN32:
                     #win32 without opengl can't do transparency,
                     #so it triggers too many warnings
                     l = log.warn
@@ -422,7 +420,7 @@ class ClientWindowBase(ClientWidgetBase):
             self.set_strut(metadata.dictget("strut", {}))
 
         if b"fullscreen-monitors" in metadata:
-            self.set_fullscreen_monitors(metadata.intlistget("fullscreen-monitors"))
+            self.set_fullscreen_monitors(metadata.inttupleget("fullscreen-monitors"))
 
         if b"shape" in metadata:
             self.set_shape(metadata.dictget("shape", {}))
@@ -431,7 +429,10 @@ class ClientWindowBase(ClientWidgetBase):
             self.set_command(metadata.strget("command"))
 
         if b"x11-property" in metadata:
-            self.set_x11_property(*metadata.listget("x11-property"))
+            self.set_x11_property(*metadata.tupleget("x11-property"))
+
+        if b"content-type" in metadata:
+            self.content_type = metadata.strget("content-type")
 
 
     def set_x11_property(self, *x11_property):
@@ -494,7 +495,7 @@ class ClientWindowBase(ClientWidgetBase):
                     if not closetoint(fsv1) or not closetoint(fsv2):
                         #the scaled value is not close to an int,
                         #so we can't honour it:
-                        geomlog("cannot honour '%s' due to scaling, scaled values are not integers: %s, %s",
+                        geomlog("cannot honour '%s' due to scaling, scaled values are not both integers: %s, %s",
                                 a, fsv1, fsv2)
                         continue
                 hints[h1], hints[h2] = sv1, sv2
@@ -506,7 +507,7 @@ class ClientWindowBase(ClientWidgetBase):
                 v = size_constraints.intpair(a)
                 if v:
                     v1, v2 = v
-                    hints[h] = float(v1*self._client.xscale)/float(v2*self._client.yscale)
+                    hints[h] = (v1*self._client.xscale)/(v2*self._client.yscale)
         #apply max-size override if needed:
         w,h = max_window_size
         if w>0 and h>0 and not self._fullscreen:
@@ -544,7 +545,7 @@ class ClientWindowBase(ClientWidgetBase):
                     pass
             #bug 2214: GTK3 on win32 gets confused if we specify a large max-size
             # and it will mess up maximizing the window
-            if not WIN32 or PYTHON2 or (maxw<32000 or maxh<32000):
+            if not WIN32 or (maxw<32000 or maxh<32000):
                 hints[b"max_width"] = maxw
                 hints[b"max_height"] = maxh
         try:
@@ -556,7 +557,7 @@ class ClientWindowBase(ClientWidgetBase):
             geomlog("set_size_constraints%s", (size_constraints, max_window_size), exc_info=True)
             geomlog.error("Error setting window hints:")
             for k,v in hints.items():
-                geomlog.error(" %s=%s", k, v)
+                geomlog.error(" %s=%s", bytestostr(k), v)
             geomlog.error(" from size constraints:")
             for k,v in size_constraints.items():
                 geomlog.error(" %s=%s", k, v)
@@ -571,7 +572,7 @@ class ClientWindowBase(ClientWidgetBase):
         for window_type in window_types:
             #win32 workaround:
             if AWT_DIALOG_WORKAROUND and window_type=="DIALOG" and self._metadata.boolget("skip-taskbar"):
-                wm_class = self._metadata.strlistget("class-instance", (None, None), 2, 2)
+                wm_class = self._metadata.strtupleget("class-instance", (None, None), 2, 2)
                 if wm_class and len(wm_class)==2 and wm_class[0] and wm_class[0].startswith("sun-awt-X11"):
                     #replace "DIALOG" with "NORMAL":
                     if "NORMAL" in window_types:
@@ -598,6 +599,7 @@ class ClientWindowBase(ClientWidgetBase):
 
     def toggle_debug(self, *_args):
         b = self._backing
+        log.info("toggling debug on backing %s for window %i", b, self._id)
         if not b:
             return
         if b.paint_box_line_width>0:
@@ -692,10 +694,6 @@ class ClientWindowBase(ClientWidgetBase):
                 self.pending_refresh.append((x, y, width, height))
             if options.intget("flush", 0)==0:
                 callbacks.append(self.after_draw_refresh)
-        if coding=="void":
-            from xpra.client.window_backing_base import fire_paint_callbacks
-            fire_paint_callbacks(callbacks)
-            return
         backing.draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
 
     def after_draw_refresh(self, success, message=""):
@@ -713,9 +711,9 @@ class ClientWindowBase(ClientWidgetBase):
         self.pending_refresh = []
         for x, y, w, h in pr:
             rx, ry, rw, rh = self._client.srect(x, y, w, h)
-            if self.window_offset:
-                rx += self.window_offset[0]
-                ry += self.window_offset[1]
+            #if self.window_offset:
+            #    rx -= self.window_offset[0]
+            #    ry -= self.window_offset[1]
             self.idle_add(self.queue_draw_area, rx, ry, rw, rh)
 
     def eos(self):
@@ -736,7 +734,7 @@ class ClientWindowBase(ClientWidgetBase):
     def can_have_spinner(self):
         if self._backing is None:
             return False
-        window_types = self._metadata.strlistget("window-type")
+        window_types = self._metadata.strtupleget("window-type")
         if not window_types:
             return False
         return ("NORMAL" in window_types) or \
