@@ -7,38 +7,42 @@
 
 import sys
 import datetime
+import platform
 from collections import deque
+from gi.repository import GLib, Gtk, Gdk
 
 from xpra.version_util import XPRA_VERSION
 from xpra.os_util import bytestostr, strtobytes, get_linux_distribution, monotonic_time
 from xpra.util import prettify_plug_name, typedict, csv, engs, iround
 from xpra.gtk_common.graph import make_graph_imagesurface
 from xpra.simple_stats import values_to_scaled_values, values_to_diff_scaled_values, to_std_unit, std_unit_dec, std_unit
-from xpra.scripts.config import python_platform
 from xpra.client import mixin_features
 from xpra.gtk_common.gtk_util import (
-    add_close_accel, label, title_box,
-    TableBuilder, imagebutton, get_preferred_size, get_gtk_version_info,
-    RELIEF_NONE, RELIEF_NORMAL, EXPAND, FILL, WIN_POS_CENTER,
-    RESPONSE_CANCEL, RESPONSE_OK, RESPONSE_CLOSE, RESPONSE_DELETE_EVENT,
-    FILE_CHOOSER_ACTION_SAVE,
+    add_close_accel, label,
+    TableBuilder, imagebutton, get_gtk_version_info,
     )
 from xpra.net.net_util import get_network_caps
-from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_glib, is_gtk3
 from xpra.log import Logger
 
 log = Logger("info")
 
-gtk = import_gtk()
-gdk = import_gdk()
-glib = import_glib()
-
 N_SAMPLES = 20      #how many sample points to show on the graphs
 SHOW_PIXEL_STATS = True
 SHOW_SOUND_STATS = True
 SHOW_RECV = True
 
 
+def title_box(label_str):
+    eb = Gtk.EventBox()
+    l = label(label_str)
+    l.modify_fg(Gtk.StateType.NORMAL, Gdk.Color(red=48*256, green=0, blue=0))
+    al = Gtk.Alignment(xalign=0.0, yalign=0.5, xscale=0.0, yscale=0.0)
+    al.set_padding(0, 0, 10, 10)
+    al.add(l)
+    eb.add(al)
+    eb.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(red=219*256, green=226*256, blue=242*256))
+    return eb
+
 def pixelstr(v):
     if v<0:
         return  "n/a"
@@ -112,10 +116,10 @@ def slabel(text="", tooltip=None, font=None):
     return l
 
 
-class SessionInfo(gtk.Window):
+class SessionInfo(Gtk.Window):
 
     def __init__(self, client, session_name, window_icon_pixbuf, conn, get_pixbuf):
-        gtk.Window.__init__(self)
+        Gtk.Window.__init__(self)
         self.client = client
         self.session_name = session_name
         self.connection = conn
@@ -123,21 +127,17 @@ class SessionInfo(gtk.Window):
         self.last_populate_statistics = 0
         self.is_closed = False
         self.get_pixbuf = get_pixbuf
-        if not self.session_name or self.session_name=="Xpra":
-            title = u"Session Info"
-        else:
-            title = u"%s: Session Info" % self.session_name
-        self.set_title(title)
+        self.set_title(self.get_window_title())
         self.set_destroy_with_parent(True)
         self.set_resizable(True)
         self.set_decorated(True)
         if window_icon_pixbuf:
             self.set_icon(window_icon_pixbuf)
-        self.set_position(WIN_POS_CENTER)
+        self.set_position(Gtk.WindowPosition.CENTER)
 
         #tables on the left in a vbox with buttons at the top:
-        self.tab_box = gtk.VBox(False, 0)
-        self.tab_button_box = gtk.HBox(True, 0)
+        self.tab_box = Gtk.VBox(False, 0)
+        self.tab_button_box = Gtk.HBox(True, 0)
         self.tabs = []          #pairs of button, table
         self.populate_cb = None
         self.tab_box.pack_start(self.tab_button_box, expand=False, fill=True, padding=0)
@@ -145,9 +145,10 @@ class SessionInfo(gtk.Window):
         #Package Table:
         tb = self.table_tab("package.png", "Software", self.populate_package)[0]
         #title row:
-        tb.attach(title_box(""), 0, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("Client"), 1, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("Server"), 2, xoptions=EXPAND|FILL, xpadding=0)
+        fillexpand = Gtk.AttachOptions.FILL | Gtk.AttachOptions.EXPAND
+        tb.attach(title_box(""), 0, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("Client"), 1, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("Server"), 2, xoptions=fillexpand, xpadding=0)
         tb.inc()
 
         def make_os_str(sys_platform, platform_release, platform_platform, platform_linux_distribution):
@@ -159,37 +160,24 @@ class SessionInfo(gtk.Window):
                 s.append(platform_platform)
             return "\n".join(s)
         distro = get_linux_distribution()
-        LOCAL_PLATFORM_NAME = make_os_str(sys.platform, python_platform.release(), python_platform.platform(), distro)
-        SERVER_PLATFORM_NAME = make_os_str(self.client._remote_platform,
-                                           self.client._remote_platform_release,
-                                           self.client._remote_platform_platform,
-                                           self.client._remote_platform_linux_distribution)
+        LOCAL_PLATFORM_NAME = make_os_str(sys.platform, platform.release(), platform.platform(), distro)
+        def cattr(name, default_value=""):
+            return getattr(self.client, name, default_value)
+        SERVER_PLATFORM_NAME = make_os_str(cattr("_remote_platform"),
+                                           cattr("_remote_platform_release"),
+                                           cattr("_remote_platform_platform"),
+                                           cattr("_remote_platform_linux_distribution"))
         tb.new_row("Operating System", slabel(LOCAL_PLATFORM_NAME), slabel(SERVER_PLATFORM_NAME))
-        scaps = self.client.server_capabilities
-        tb.new_row("Xpra", slabel(XPRA_VERSION), slabel(self.client._remote_version or "unknown"))
-        cl_rev, cl_ch, cl_date = "unknown", "", ""
+        tb.new_row("Xpra", slabel(XPRA_VERSION), slabel(cattr("_remote_version", "unknown")))
         try:
             from xpra.build_info import BUILD_DATE as cl_date, BUILD_TIME as cl_time
             from xpra.src_info import REVISION as cl_rev, LOCAL_MODIFICATIONS as cl_ch      #@UnresolvedImport
-        except:
-            pass
+        except ImportError:
+            cl_rev, cl_ch, cl_date, cl_time = "unknown", "", "", ""
         def make_version_str(version):
             if version and isinstance(version, (tuple, list)):
                 version = ".".join(bytestostr(x) for x in version)
             return bytestostr(version or "unknown")
-        def server_info(*prop_names):
-            for x in prop_names:
-                k = strtobytes(x)
-                v = dictlook(scaps, k)
-                #log("server_info%s dictlook(%s)=%s", prop_names, k, v)
-                if v is not None:
-                    return v
-                v = dictlook(self.client.server_last_info, k)
-                if v is not None:
-                    return v
-            return None
-        def server_version_info(prop_name):
-            return make_version_str(server_info(prop_name))
         def make_revision_str(rev, changes):
             try:
                 cint = int(changes)
@@ -202,25 +190,22 @@ class SessionInfo(gtk.Window):
                 return bytestostr(date)
             return "%s %s" % (bytestostr(date), bytestostr(time))
         tb.new_row("Revision", slabel(make_revision_str(cl_rev, cl_ch)),
-                               slabel(make_revision_str(self.client._remote_revision, server_version_info("build.local_modifications"))))
+                               slabel(make_revision_str(cattr("_remote_revision"), cattr("_remote_modifications"))))
         tb.new_row("Build date", slabel(make_datetime(cl_date, cl_time)),
-                                 slabel(make_datetime(server_info("build_date", "build.date"), server_info("build.time"))))
+                                 slabel(make_datetime(cattr("_remote_build_date"), cattr("_remote_build_time"))))
         gtk_version_info = get_gtk_version_info()
         def client_vinfo(prop, fallback="unknown"):
             s = make_version_str(newdictlook(gtk_version_info, (prop, "version"), fallback))
             return slabel(s)
-        def server_vinfo(prop):
-            k = "%s.version" % prop
-            return slabel(server_version_info(k))
+        def server_vinfo(lib):
+            return slabel(make_version_str(self.client._remote_lib_versions.get(lib, "")))
         tb.new_row("Glib",      client_vinfo("glib"),       server_vinfo("glib"))
-        tb.new_row("PyGlib",    client_vinfo("pyglib"),     server_vinfo("pyglib"))
         tb.new_row("Gobject",   client_vinfo("gobject"),    server_vinfo("gobject"))
-        tb.new_row("PyGTK",     client_vinfo("pygtk", ""),  server_vinfo("pygtk"))
         tb.new_row("GTK",       client_vinfo("gtk"),        server_vinfo("gtk"))
         tb.new_row("GDK",       client_vinfo("gdk"),        server_vinfo("gdk"))
         tb.new_row("Cairo",     client_vinfo("cairo"),      server_vinfo("cairo"))
         tb.new_row("Pango",     client_vinfo("pango"),      server_vinfo("pango"))
-        tb.new_row("Python", slabel(python_platform.python_version()), slabel(server_version_info("server.python.version")))
+        tb.new_row("Python", slabel(platform.python_version()), slabel(cattr("_remote_python_version")))
 
         try:
             from xpra.sound.wrapper import query_sound
@@ -228,37 +213,39 @@ class SessionInfo(gtk.Window):
         except Exception:
             log("cannot load sound information: %s", exc_info=True)
             props = typedict()
-        gst_version = props.strlistget("gst.version")
-        pygst_version = props.strlistget("pygst.version")
-        tb.new_row("GStreamer", slabel(make_version_str(gst_version)), slabel(server_version_info("sound.gst.version")))
-        tb.new_row("pygst", slabel(make_version_str(pygst_version)), slabel(server_version_info("sound.pygst.version")))
+        gst_version = props.strtupleget("gst.version")
+        pygst_version = props.strtupleget("pygst.version")
+        tb.new_row("GStreamer", slabel(make_version_str(gst_version)), server_vinfo("sound.gst"))
+        tb.new_row("pygst", slabel(make_version_str(pygst_version)), server_vinfo("sound.pygst"))
         def gllabel(prop="opengl", default_value="n/a"):
             return slabel(make_version_str(self.client.opengl_props.get(prop, default_value)))
-        tb.new_row("OpenGL", gllabel("opengl", "n/a"), slabel(server_version_info("opengl.opengl")))
-        tb.new_row("OpenGL Vendor", gllabel("vendor", ""), slabel(server_version_info("opengl.vendor")))
-        tb.new_row("PyOpenGL", gllabel("pyopengl", "n/a"), slabel(server_version_info("opengl.pyopengl")))
+        def sgllabel(prop="opengl", default_value="n/a"):
+            return slabel(make_version_str(typedict(self.client.server_opengl or {}).strget(prop, default_value)))
+        tb.new_row("OpenGL", gllabel("opengl", "n/a"), sgllabel("opengl"))
+        tb.new_row("OpenGL Vendor", gllabel("vendor", ""), sgllabel("vendor"))
+        tb.new_row("PyOpenGL", gllabel("pyopengl", "n/a"), sgllabel("pyopengl"))
 
         # Features Table:
         vbox = self.vbox_tab("features.png", "Features", self.populate_features)
         #add top table:
         tb = TableBuilder(rows=1, columns=2, row_spacings=15)
         table = tb.get_table()
-        al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=1.0)
+        al = Gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=1.0)
         al.add(table)
         vbox.pack_start(al, expand=True, fill=False, padding=10)
         #top table contents:
-        randr_box = gtk.HBox(False, 20)
+        randr_box = Gtk.HBox(False, 20)
         self.server_randr_label = slabel()
-        self.server_randr_icon = gtk.Image()
+        self.server_randr_icon = Gtk.Image()
         randr_box.add(self.server_randr_icon)
         randr_box.add(self.server_randr_label)
         tb.new_row("RandR Support", randr_box)
         self.client_display = slabel()
         tb.new_row("Client Display", self.client_display)
-        opengl_box = gtk.HBox(False, 20)
+        opengl_box = Gtk.HBox(False, 20)
         self.client_opengl_label = slabel()
         self.client_opengl_label.set_line_wrap(True)
-        self.client_opengl_icon = gtk.Image()
+        self.client_opengl_icon = Gtk.Image()
         opengl_box.add(self.client_opengl_icon)
         opengl_box.add(self.client_opengl_label)
         tb.new_row("Client OpenGL", opengl_box)
@@ -266,28 +253,32 @@ class SessionInfo(gtk.Window):
         tb.new_row("OpenGL Mode", self.opengl_buffering)
         self.window_rendering = slabel()
         tb.new_row("Window Rendering", self.window_rendering)
-        self.server_mmap_icon = gtk.Image()
-        tb.new_row("Memory Mapped Transfers", self.server_mmap_icon)
-        self.server_clipboard_icon = gtk.Image()
-        tb.new_row("Clipboard", self.server_clipboard_icon)
-        self.server_notifications_icon = gtk.Image()
-        tb.new_row("Notifications", self.server_notifications_icon)
-        self.server_bell_icon = gtk.Image()
-        tb.new_row("Bell", self.server_bell_icon)
-        self.server_cursors_icon = gtk.Image()
-        tb.new_row("Cursors", self.server_cursors_icon)
+        if mixin_features.mmap:
+            self.server_mmap_icon = Gtk.Image()
+            tb.new_row("Memory Mapped Transfers", self.server_mmap_icon)
+        if mixin_features.clipboard:
+            self.server_clipboard_icon = Gtk.Image()
+            tb.new_row("Clipboard", self.server_clipboard_icon)
+        if mixin_features.notifications:
+            self.server_notifications_icon = Gtk.Image()
+            tb.new_row("Notifications", self.server_notifications_icon)
+        if mixin_features.windows:
+            self.server_bell_icon = Gtk.Image()
+            tb.new_row("Bell", self.server_bell_icon)
+            self.server_cursors_icon = Gtk.Image()
+            tb.new_row("Cursors", self.server_cursors_icon)
 
         # Codecs Table:
         vbox = self.vbox_tab("encoding.png", "Codecs", self.populate_codecs)
         tb = TableBuilder(rows=1, columns=2, col_spacings=0, row_spacings=10)
         table = tb.get_table()
-        al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=1.0)
+        al = Gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=1.0)
         al.add(table)
         vbox.pack_start(al, expand=True, fill=False, padding=10)
         #table headings:
-        tb.attach(title_box(""), 0, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("Client"), 1, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("Server"), 2, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box(""), 0, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("Client"), 1, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("Server"), 2, xoptions=fillexpand, xpadding=0)
         tb.inc()
         #table contents:
         self.client_encodings_label = slabel()
@@ -296,33 +287,33 @@ class SessionInfo(gtk.Window):
         self.server_encodings_label.set_line_wrap(True)
         tb.new_row("Picture Encodings",
                    self.client_encodings_label, self.server_encodings_label,
-                   xoptions=FILL|EXPAND, yoptions=FILL|EXPAND)
+                   xoptions=fillexpand, yoptions=fillexpand)
         self.client_speaker_codecs_label = slabel()
         self.client_speaker_codecs_label.set_line_wrap(True)
         self.server_speaker_codecs_label = slabel()
         self.server_speaker_codecs_label.set_line_wrap(True)
         tb.new_row("Speaker Codecs",
                    self.client_speaker_codecs_label, self.server_speaker_codecs_label,
-                   xoptions=FILL|EXPAND, yoptions=FILL|EXPAND)
+                   xoptions=fillexpand, yoptions=fillexpand)
         self.client_microphone_codecs_label = slabel()
         self.client_microphone_codecs_label.set_line_wrap(True)
         self.server_microphone_codecs_label = slabel()
         self.server_microphone_codecs_label.set_line_wrap(True)
         tb.new_row("Microphone Codecs",
                    self.client_microphone_codecs_label, self.server_microphone_codecs_label,
-                   xoptions=FILL|EXPAND, yoptions=FILL|EXPAND)
+                   xoptions=fillexpand, yoptions=fillexpand)
         self.client_packet_encoders_label = slabel()
         self.client_packet_encoders_label.set_line_wrap(True)
         self.server_packet_encoders_label = slabel()
         self.server_packet_encoders_label.set_line_wrap(True)
         tb.new_row("Packet Encoders",
                    self.client_packet_encoders_label, self.server_packet_encoders_label,
-                   xoptions=FILL|EXPAND, yoptions=FILL|EXPAND)
+                   xoptions=fillexpand, yoptions=fillexpand)
         self.client_packet_compressors_label = slabel()
         self.server_packet_compressors_label = slabel()
         tb.new_row("Packet Compressors",
                    self.client_packet_compressors_label, self.server_packet_compressors_label,
-                   xoptions=FILL|EXPAND, yoptions=FILL|EXPAND)
+                   xoptions=fillexpand, yoptions=fillexpand)
 
         # Connection Table:
         tb = self.table_tab("connect.png", "Connection", self.populate_connection)[0]
@@ -330,11 +321,9 @@ class SessionInfo(gtk.Window):
             tb.new_row("Server Endpoint", slabel(self.connection.target))
         if mixin_features.display and self.client.server_display:
             tb.new_row("Server Display", slabel(prettify_plug_name(self.client.server_display)))
-        hostname = scaps.strget("hostname")
-        if hostname:
-            tb.new_row("Server Hostname", slabel(hostname))
-        if self.client.server_platform:
-            tb.new_row("Server Platform", slabel(self.client.server_platform))
+        tb.new_row("Server Hostname", slabel(cattr("_remote_hostname")))
+        if cattr("server_platform"):
+            tb.new_row("Server Platform", slabel(cattr("server_platform")))
         self.server_load_label = slabel()
         tb.new_row("Server Load", self.server_load_label, label_tooltip="Average over 1, 5 and 15 minutes")
         self.session_started_label = slabel()
@@ -367,12 +356,12 @@ class SessionInfo(gtk.Window):
         # Details:
         tb, stats_box = self.table_tab("browse.png", "Statistics", self.populate_statistics)
         tb.widget_xalign = 1.0
-        tb.attach(title_box(""), 0, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("Latest"), 1, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("Minimum"), 2, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("Average"), 3, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("90 percentile"), 4, xoptions=EXPAND|FILL, xpadding=0)
-        tb.attach(title_box("Maximum"), 5, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box(""), 0, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("Latest"), 1, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("Minimum"), 2, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("Average"), 3, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("90 percentile"), 4, xoptions=fillexpand, xpadding=0)
+        tb.attach(title_box("Maximum"), 5, xoptions=fillexpand, xpadding=0)
         tb.inc()
 
         def maths_labels():
@@ -420,15 +409,15 @@ class SessionInfo(gtk.Window):
             wtb = TableBuilder()
             stats_box.add(wtb.get_table())
             #title row:
-            wtb.attach(title_box(""), 0, xoptions=EXPAND|FILL, xpadding=0)
-            wtb.attach(title_box("Regular"), 1, xoptions=EXPAND|FILL, xpadding=0)
-            wtb.attach(title_box("Transient"), 2, xoptions=EXPAND|FILL, xpadding=0)
-            wtb.attach(title_box("Trays"), 3, xoptions=EXPAND|FILL, xpadding=0)
+            wtb.attach(title_box(""), 0, xoptions=fillexpand, xpadding=0)
+            wtb.attach(title_box("Regular"), 1, xoptions=fillexpand, xpadding=0)
+            wtb.attach(title_box("Transient"), 2, xoptions=fillexpand, xpadding=0)
+            wtb.attach(title_box("Trays"), 3, xoptions=fillexpand, xpadding=0)
             if self.client.client_supports_opengl:
-                wtb.attach(title_box("OpenGL"), 4, xoptions=EXPAND|FILL, xpadding=0)
+                wtb.attach(title_box("OpenGL"), 4, xoptions=fillexpand, xpadding=0)
             wtb.inc()
 
-            wtb.attach(slabel("Windows:"), 0, xoptions=EXPAND|FILL, xpadding=0)
+            wtb.attach(slabel("Windows:"), 0, xoptions=fillexpand, xpadding=0)
             self.windows_managed_label = slabel()
             wtb.attach(self.windows_managed_label, 1)
             self.transient_managed_label = slabel()
@@ -442,10 +431,10 @@ class SessionInfo(gtk.Window):
             #add encoder info:
             etb = TableBuilder()
             stats_box.add(etb.get_table())
-            self.encoder_info_box = gtk.HBox(spacing=4)
+            self.encoder_info_box = Gtk.HBox(spacing=4)
             etb.new_row("Window Encoders", self.encoder_info_box)
 
-        self.graph_box = gtk.VBox(False, 10)
+        self.graph_box = Gtk.VBox(False, 10)
         self.add_tab("statistics.png", "Graphs", self.populate_graphs, self.graph_box)
         bandwidth_label = "Bandwidth used"
         if SHOW_PIXEL_STATS:
@@ -458,8 +447,8 @@ class SessionInfo(gtk.Window):
             self.sound_queue_graph = None
         self.connect("realize", self.populate_graphs)
         self.pixel_in_data = deque(maxlen=N_SAMPLES+4)
-        self.net_in_bytecount = deque(maxlen=N_SAMPLES+4)
-        self.net_out_bytecount = deque(maxlen=N_SAMPLES+4)
+        self.net_in_bitcount = deque(maxlen=N_SAMPLES+4)
+        self.net_out_bitcount = deque(maxlen=N_SAMPLES+4)
         self.sound_in_bitcount = deque(maxlen=N_SAMPLES+4)
         self.sound_out_bitcount = deque(maxlen=N_SAMPLES+4)
         self.sound_out_queue_min = deque(maxlen=N_SAMPLES*10+4)
@@ -468,8 +457,6 @@ class SessionInfo(gtk.Window):
 
         self.set_border_width(15)
         self.add(self.tab_box)
-        if not is_gtk3():
-            self.set_geometry_hints(self.tab_box)
         def window_deleted(*_args):
             self.is_closed = True
         self.connect('delete_event', window_deleted)
@@ -478,24 +465,39 @@ class SessionInfo(gtk.Window):
         self.init_counters()
         self.populate()
         self.populate_all()
-        glib.timeout_add(1000, self.populate)
-        glib.timeout_add(100, self.populate_tab)
+        GLib.timeout_add(1000, self.populate)
+        GLib.timeout_add(100, self.populate_tab)
         if mixin_features.audio and SHOW_SOUND_STATS:
-            glib.timeout_add(100, self.populate_sound_stats)
+            GLib.timeout_add(100, self.populate_sound_stats)
         add_close_accel(self, self.destroy)
 
 
+    def get_window_title(self):
+        t = ["Session Info"]
+        c = self.client
+        if c:
+            if c.session_name or c.server_session_name:
+                t.append(c.session_name or c.server_session_name)
+            p = c._protocol
+            if p:
+                conn = getattr(p, "_conn", None)
+                if conn:
+                    cinfo = conn.get_info()
+                    t.append(cinfo.get("endpoint", bytestostr(conn.target)))
+        v = " - ".join(str(x) for x in t)
+        return v
+
     def table_tab(self, icon_filename, title, populate_cb):
         tb = TableBuilder()
         table = tb.get_table()
         vbox = self.vbox_tab(icon_filename, title, populate_cb)
-        al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=1.0)
+        al = Gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=1.0)
         al.add(table)
         vbox.pack_start(al, expand=True, fill=True, padding=20)
         return tb, vbox
 
     def vbox_tab(self, icon_filename, title, populate_cb):
-        vbox = gtk.VBox(False, 0)
+        vbox = Gtk.VBox(False, 0)
         self.add_tab(icon_filename, title, populate_cb, contents=vbox)
         return vbox
 
@@ -506,7 +508,7 @@ class SessionInfo(gtk.Window):
             self.show_tab(contents)
         button = imagebutton(title, icon, clicked_callback=show_tab)
         button.connect("clicked", show_tab)
-        button.set_relief(RELIEF_NONE)
+        button.set_relief(Gtk.ReliefStyle.NONE)
         self.tab_button_box.add(button)
         self.tabs.append((title, button, contents, populate_cb))
 
@@ -515,11 +517,11 @@ class SessionInfo(gtk.Window):
         for _, b, t, p_cb in self.tabs:
             if t==table:
                 button = b
-                b.set_relief(RELIEF_NORMAL)
+                b.set_relief(Gtk.ReliefStyle.NORMAL)
                 b.grab_focus()
                 self.populate_cb = p_cb
             else:
-                b.set_relief(RELIEF_NONE)
+                b.set_relief(Gtk.ReliefStyle.NONE)
         assert button
         for x in self.tab_box.get_children():
             if x!=self.tab_button_box:
@@ -556,15 +558,12 @@ class SessionInfo(gtk.Window):
                 p_cb()
 
     def add_graph_button(self, tooltip, click_cb):
-        button = gtk.EventBox()
+        button = Gtk.EventBox()
         def set_cursor(widget):
-            if is_gtk3():
-                cursor = gdk.Cursor.new(gdk.CursorType.BASED_ARROW_DOWN)
-            else:
-                cursor = gdk.Cursor(gdk.BASED_ARROW_DOWN)
+            cursor = Gdk.Cursor.new(Gdk.CursorType.BASED_ARROW_DOWN)
             widget.get_window().set_cursor(cursor)
         button.connect("realize", set_cursor)
-        graph = gtk.Image()
+        graph = Gtk.Image()
         graph.set_size_request(0, 0)
         button.connect("button_press_event", click_cb, graph)
         button.add(graph)
@@ -606,8 +605,8 @@ class SessionInfo(gtk.Window):
         self.show_opengl_state()
         self.show_window_renderers()
         #record bytecount every second:
-        self.net_in_bytecount.append(conn.input_bytecount)
-        self.net_out_bytecount.append(conn.output_bytecount)
+        self.net_in_bitcount.append(conn.input_bytecount*8)
+        self.net_out_bitcount.append(conn.output_bytecount*8)
         if mixin_features.audio and SHOW_SOUND_STATS:
             if self.client.sound_in_bytecount>0:
                 self.sound_in_bitcount.append(self.client.sound_in_bytecount * 8)
@@ -739,7 +738,7 @@ class SessionInfo(gtk.Window):
             renderers.setdefault(window.get_backing_class(), []).append(wid)
         for bclass, windows in renderers.items():
             wr.append("%s (%i)" % (bclass.__name__.replace("Backing", ""), len(windows)))
-        self.window_rendering.set_text("GTK%s: %s" % (["2","3"][is_gtk3()], csv(wr)))
+        self.window_rendering.set_text("GTK3: %s" % csv(wr))
 
     def populate_features(self):
         size_info = ""
@@ -765,17 +764,20 @@ class SessionInfo(gtk.Window):
         self.client_display.set_text(display_info)
         self.bool_icon(self.client_opengl_icon, self.client.client_supports_opengl)
 
-        scaps = self.client.server_capabilities
         self.show_window_renderers()
-        self.bool_icon(self.server_mmap_icon, self.client.mmap_enabled)
-        self.bool_icon(self.server_clipboard_icon,      scaps.boolget("clipboard", False))
-        self.bool_icon(self.server_notifications_icon,  scaps.boolget("notifications", False))
-        self.bool_icon(self.server_bell_icon,           scaps.boolget("bell", False))
-        self.bool_icon(self.server_cursors_icon,        scaps.boolget("cursors", False))
+        if mixin_features.mmap:
+            self.bool_icon(self.server_mmap_icon, self.client.mmap_enabled)
+        if mixin_features.clipboard:
+            self.bool_icon(self.server_clipboard_icon, self.client.server_clipboard)
+        if mixin_features.notifications:
+            self.bool_icon(self.server_notifications_icon, self.client.server_notifications)
+        if mixin_features.windows:
+            self.bool_icon(self.server_bell_icon, self.client.server_bell)
+            self.bool_icon(self.server_cursors_icon, self.client.server_cursors)
 
     def populate_codecs(self):
         #clamp the large labels so they will overflow vertically:
-        w = get_preferred_size(self.tab_box)[0]
+        w = self.tab_box.get_preferred_width()[0]
         lw = max(200, int(w//2.5))
         self.client_encodings_label.set_size_request(lw, -1)
         self.server_encodings_label.set_size_request(lw, -1)
@@ -783,19 +785,17 @@ class SessionInfo(gtk.Window):
         self.server_speaker_codecs_label.set_size_request(lw, -1)
         self.client_microphone_codecs_label.set_size_request(lw, -1)
         self.server_microphone_codecs_label.set_size_request(lw, -1)
-        self.client_packet_encoders_label.set_size_request(lw, -1)
-        self.server_packet_encoders_label.set_size_request(lw, -1)
         #sound/video codec table:
-        scaps = self.client.server_capabilities
         def codec_info(enabled, codecs):
             if not enabled:
                 return "n/a"
-            return ", ".join(codecs or [])
+            return ", ".join(codecs or ())
         if mixin_features.audio:
-            self.server_speaker_codecs_label.set_text(codec_info(scaps.boolget("sound.send", False), scaps.strlistget("sound.encoders", [])))
-            self.client_speaker_codecs_label.set_text(codec_info(self.client.speaker_allowed, self.client.speaker_codecs))
-            self.server_microphone_codecs_label.set_text(codec_info(scaps.boolget("sound.receive", False), scaps.strlistget("sound.decoders", [])))
-            self.client_microphone_codecs_label.set_text(codec_info(self.client.microphone_allowed, self.client.microphone_codecs))
+            c = self.client
+            self.server_speaker_codecs_label.set_text(codec_info(c.server_sound_send, c.server_sound_encoders))
+            self.client_speaker_codecs_label.set_text(codec_info(c.speaker_allowed, c.speaker_codecs))
+            self.server_microphone_codecs_label.set_text(codec_info(c.server_sound_receive, c.server_sound_decoders))
+            self.client_microphone_codecs_label.set_text(codec_info(c.microphone_allowed, c.microphone_codecs))
         def encliststr(v):
             v = list(v)
             try:
@@ -803,24 +803,22 @@ class SessionInfo(gtk.Window):
             except ValueError:
                 pass
             return csv(sorted(v))
-        se = scaps.strlistget("encodings.core", scaps.strlistget("encodings"))
+        se = ()
+        if mixin_features.encoding:
+            se = self.client.server_core_encodings
         self.server_encodings_label.set_text(encliststr(se))
         if mixin_features.encoding:
             self.client_encodings_label.set_text(encliststr(self.client.get_core_encodings()))
         else:
             self.client_encodings_label.set_text("n/a")
 
-        def get_encoder_list(caps):
-            from xpra.net import packet_encoding
-            return [x for x in packet_encoding.ALL_ENCODERS if typedict(caps).rawget(x)]
-        self.client_packet_encoders_label.set_text(", ".join(get_encoder_list(get_network_caps())))
-        self.server_packet_encoders_label.set_text(", ".join(get_encoder_list(self.client.server_capabilities)))
+        from xpra.net.packet_encoding import get_enabled_encoders
+        self.client_packet_encoders_label.set_text(csv(get_enabled_encoders()))
+        self.server_packet_encoders_label.set_text(csv(self.client.server_packet_encoders))
 
-        def get_compressor_list(caps):
-            from xpra.net import compression
-            return [x for x in compression.ALL_COMPRESSORS if typedict(caps).rawget(x)]
-        self.client_packet_compressors_label.set_text(", ".join(get_compressor_list(get_network_caps())))
-        self.server_packet_compressors_label.set_text(", ".join(get_compressor_list(self.client.server_capabilities)))
+        from xpra.net.compression import get_enabled_compressors
+        self.client_packet_compressors_label.set_text(csv(get_enabled_compressors()))
+        self.server_packet_compressors_label.set_text(csv(self.client.server_packet_compressors))
         return False
 
     def populate_connection(self):
@@ -1093,22 +1091,15 @@ class SessionInfo(gtk.Window):
         h = surface.get_height()
         graph.set_size_request(w, h)
         graph.surface = surface
-        if is_gtk3():
-            graph.set_from_surface(surface)
-        else:
-            pixmap = gdk.Pixmap(None, w, h, 24)
-            context = pixmap.cairo_create()
-            context.set_source_surface(surface)
-            context.paint()
-            graph.set_from_pixmap(pixmap, None)
+        graph.set_from_surface(surface)
 
     def populate_graphs(self, *_args):
         #older servers have 'batch' at top level,
         #newer servers store it under client
         self.client.send_info_request("network", "damage", "state", "batch", "client")
         box = self.tab_box
-        h = get_preferred_size(box)[1]
-        bh = get_preferred_size(self.tab_button_box)[1]
+        h = box.get_preferred_height()[0]
+        bh = self.tab_button_box.get_preferred_height()[0]
         if h<=0:
             return True
         start_x_offset = min(1.0, (monotonic_time()-self.last_populate_time)*0.95)
@@ -1122,7 +1113,7 @@ class SessionInfo(gtk.Window):
         h = min(maxh//ngraphs, max(H, (h-bh-20)//ngraphs, (rect.height-bh-20)//ngraphs))
         #bandwidth graph:
         labels, datasets = [], []
-        if self.net_in_bytecount and self.net_out_bytecount:
+        if self.net_in_bitcount and self.net_out_bitcount:
             def unit(scale):
                 if scale==1:
                     return ""
@@ -1130,13 +1121,13 @@ class SessionInfo(gtk.Window):
                 if value==1:
                     return str(unit)
                 return "x%s%s" % (int(value), unit)
-            net_in_scale, net_in_data = values_to_diff_scaled_values(tuple(self.net_in_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
-            net_out_scale, net_out_data = values_to_diff_scaled_values(tuple(self.net_out_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+            net_in_scale, net_in_data = values_to_diff_scaled_values(tuple(self.net_in_bitcount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+            net_out_scale, net_out_data = values_to_diff_scaled_values(tuple(self.net_out_bitcount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
             if SHOW_RECV:
-                labels += ["recv %sB/s" % unit(net_in_scale), "sent %sB/s" % unit(net_out_scale)]
+                labels += ["recv %sb/s" % unit(net_in_scale), "sent %sb/s" % unit(net_out_scale)]
                 datasets += [net_in_data, net_out_data]
             else:
-                labels += ["recv %sB/s" % unit(net_in_scale)]
+                labels += ["recv %sb/s" % unit(net_in_scale)]
                 datasets += [net_in_data]
         if mixin_features.windows and SHOW_PIXEL_STATS and self.client.windows_enabled:
             pixel_scale, in_pixels = values_to_scaled_values(tuple(self.pixel_in_data)[3:N_SAMPLES+4], min_scaled_value=100)
@@ -1210,12 +1201,12 @@ class SessionInfo(gtk.Window):
 
     def save_graph(self, _ebox, btn, graph):
         log("save_graph%s", (btn, graph))
-        chooser = gtk.FileChooserDialog("Save graph as a PNG image",
-                                    parent=self, action=FILE_CHOOSER_ACTION_SAVE,
-                                    buttons=(gtk.STOCK_CANCEL, RESPONSE_CANCEL, gtk.STOCK_SAVE, RESPONSE_OK))
+        chooser = Gtk.FileChooserDialog("Save graph as a PNG image",
+                                    parent=self, action=Gtk.FileChooserAction.SAVE,
+                                    buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
         chooser.set_select_multiple(False)
-        chooser.set_default_response(RESPONSE_OK)
-        file_filter = gtk.FileFilter()
+        chooser.set_default_response(Gtk.ResponseType.OK)
+        file_filter = Gtk.FileFilter()
         file_filter.set_name("PNG")
         file_filter.add_pattern("*.png")
         chooser.add_filter(file_filter)
@@ -1223,14 +1214,22 @@ class SessionInfo(gtk.Window):
         filenames = chooser.get_filenames()
         chooser.hide()
         chooser.destroy()
-        if response == RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             if len(filenames)==1:
                 filename = filenames[0]
+                if not filename.lower().endswith(".png"):
+                    filename += ".png"
                 surface = graph.surface
                 log("saving surface %s to %s", surface, filename)
-                with open(filename, "wb") as f:
-                    surface.write_to_png(f)
-        elif response in (RESPONSE_CANCEL, RESPONSE_CLOSE, RESPONSE_DELETE_EVENT):
+                from io import BytesIO
+                b = BytesIO()
+                surface.write_to_png(b)
+                def save_file():
+                    with open(filename, "wb") as f:
+                        f.write(b.getvalue())
+                from xpra.make_thread import start_thread
+                start_thread(save_file, "save-graph")
+        elif response in (Gtk.ResponseType.CANCEL, Gtk.ResponseType.CLOSE, Gtk.ResponseType.DELETE_EVENT):
             log("closed/cancelled")
         else:
             log.warn("unknown chooser response: %d" % response)
@@ -1238,5 +1237,5 @@ class SessionInfo(gtk.Window):
     def destroy(self, *args):
         log("SessionInfo.destroy(%s) is_closed=%s", args, self.is_closed)
         self.is_closed = True
-        gtk.Window.destroy(self)
+        Gtk.Window.destroy(self)
         log("SessionInfo.destroy(%s) done", args)
