@@ -587,9 +587,9 @@ class TopSessionClient(MonitorXpraClient):
             if isinstance(v, (tuple, list)):
                 return sep.join(bytestostr(x) for x in v)
             return bytestostr(v)
-        if not gli.boolget("enabled", True):
-            return "OpenGL disabled %s" % gli.strget("message")
-        gl_info = "OpenGL %s enabled: %s" % (strget("opengl"), gli.strget("renderer") or gli.strget("vendor"))
+        if not gli.boolget("enabled", False):
+            return "OpenGL disabled %s" % gli.strget("message", "")
+        gl_info = "OpenGL %s enabled: %s" % (strget("opengl", "."), gli.strget("renderer") or gli.strget("vendor"))
         depth = gli.intget("depth")
         if depth not in (0, 24):
             gl_info += ", %ibits" % depth
@@ -615,7 +615,7 @@ class TopSessionClient(MonitorXpraClient):
             window_ids = ()    #no longer used or supported by servers
             self.send("info-request", [self.uuid], window_ids, categories)
         if not self.info_timer:
-            self.info_timer = self.timeout_add(REFRESH_RATE+2, self.info_timeout)
+            self.info_timer = self.timeout_add((REFRESH_RATE+2)*1000, self.info_timeout)
         return True
 
     def init_packet_handlers(self):
