@@ -17,7 +17,7 @@ from ctypes.wintypes import (
 from ctypes.wintypes import (
     HANDLE, LPSTR, LPCWSTR, UINT, INT, BOOL, WORD, HGDIOBJ,
     LONG, LPVOID, HBITMAP, LPCSTR, LPWSTR, HWINSTA,
-    HINSTANCE, HMENU, ULONG, HHOOK, MSG,
+    HINSTANCE, HMENU, ULONG,
     )
 #imported from this module but not used here:
 assert GetLastError
@@ -31,7 +31,6 @@ PBYTE = POINTER(BYTE)
 HCURSOR = HANDLE
 HICON = HANDLE
 HBRUSH = HANDLE
-LPMSG = POINTER(MSG)
 
 class CURSORINFO(Structure):
     _fields_ = [
@@ -220,6 +219,12 @@ MultiByteToWideChar.argtypes =  [UINT, DWORD, LPCSTR, c_int, LPWSTR, c_int]
 WTSGetActiveConsoleSessionId = kernel32.WTSGetActiveConsoleSessionId
 WTSGetActiveConsoleSessionId.restype = DWORD
 WTSGetActiveConsoleSessionId.argtypes = []
+QueryFullProcessImageNameA = kernel32.QueryFullProcessImageNameA
+QueryFullProcessImageNameA.restype = BOOL
+QueryFullProcessImageNameA.argtypes = [HANDLE, DWORD, LPSTR, PDWORD]
+OpenProcess = kernel32.OpenProcess
+OpenProcess.restype = HANDLE
+OpenProcess.argtypes = [DWORD, BOOL, DWORD]
 
 user32 = WinDLL("user32", use_last_error=True)
 RegisterClassExA = user32.RegisterClassExA
@@ -235,7 +240,7 @@ UnregisterClassW.restype = BOOL
 UnregisterClassW.argtypes = [LPCWSTR, HINSTANCE]
 CreateWindowExA = user32.CreateWindowExA
 CreateWindowExA.restype = HWND
-CreateWindowExA.argtypes = [DWORD, ATOM, LPCWSTR, DWORD, c_int, c_int, c_int, c_int, HWND, HMENU, HINSTANCE, LPVOID]
+#CreateWindowExA.argtypes = [DWORD, ATOM, LPCTSTR, DWORD, c_int, c_int, c_int, c_int, HWND, HMENU, HINSTANCE, LPVOID]
 CreateWindowExW = user32.CreateWindowExW
 CreateWindowExW.restype = HWND
 CreateWindowExW.argtypes = [DWORD, ATOM, LPCWSTR, DWORD, c_int, c_int, c_int, c_int, HWND, HMENU, HINSTANCE, LPVOID]
@@ -243,11 +248,7 @@ DestroyWindow = user32.DestroyWindow
 DestroyWindow.restype = BOOL
 DestroyWindow.argtypes = [HWND]
 DefWindowProcA = user32.DefWindowProcA
-DefWindowProcA.argtypes = [HWND, UINT, WPARAM, LPARAM]
-DefWindowProcA.restype = INT
 DefWindowProcW = user32.DefWindowProcW
-DefWindowProcW.argtypes = [HWND, UINT, WPARAM, LPARAM]
-DefWindowProcW.restype = INT
 MessageBoxA = user32.MessageBoxA
 MessageBoxA.restype = INT
 MessageBoxA.argtypes = [HWND, LPCTSTR, LPCTSTR, UINT]
@@ -285,20 +286,10 @@ GetMonitorInfoW.argtypes = [HMONITOR, POINTER(MONITORINFOEX)]
 GetMonitorInfoW.restype = BOOL
 UnhookWindowsHookEx = user32.UnhookWindowsHookEx
 CallNextHookEx = user32.CallNextHookEx
-CallNextHookEx.restype = LRESULT
-CallNextHookEx.argtyps = [HHOOK, INT, WPARAM, LPARAM]
 SetWindowsHookExA = user32.SetWindowsHookExA
-SetWindowsHookExA.restype = HHOOK
-SetWindowsHookExA.argtypes = [INT, HANDLE, HINSTANCE, DWORD]
 GetMessageA = user32.GetMessageA
-GetMessageA.restype = BOOL
-GetMessageA.argtypes = [LPMSG, HWND, UINT, UINT]
 TranslateMessage = user32.TranslateMessage
-TranslateMessage.restype = BOOL
-TranslateMessage.argtypes = [LPMSG]
 DispatchMessageA = user32.DispatchMessageA
-DispatchMessageA.restype = LRESULT
-DispatchMessageA.argtypes = [LPMSG]
 MapVirtualKeyW = user32.MapVirtualKeyW
 GetKeyboardState = user32.GetKeyboardState
 GetKeyboardState.argtypes = [PBYTE]
@@ -327,6 +318,7 @@ IsWindowVisible = user32.IsWindowVisible
 GetWindowTextLengthW = user32.GetWindowTextLengthW
 GetWindowTextW = user32.GetWindowTextW
 GetWindowThreadProcessId = user32.GetWindowThreadProcessId
+GetWindowThreadProcessId.argtypes = [HWND, LPDWORD]
 GetWindowThreadProcessId.restype = DWORD
 GetDesktopWindow = user32.GetDesktopWindow
 GetDesktopWindow.restype = HWND
@@ -397,7 +389,19 @@ CreateWindowStationA.argtypes = [LPCSTR, DWORD, ACCESS_MASK, LPSECURITY_ATTRIBUT
 CloseWindowStation = user32.CloseWindowStation
 CloseWindowStation.restype = BOOL
 CloseWindowStation.argtypes = [HWINSTA]
+WINSTAENUMPROCA = WINFUNCTYPE(BOOL, LPSTR, LPARAM)
+EnumWindowStationsA = user32.EnumWindowStationsA
+EnumWindowStationsA.restype = BOOL
+EnumWindowStationsA.argtypes = [WINSTAENUMPROCA, LPARAM]
 HDESK = HANDLE
+CreateDesktopA = user32.CreateDesktopA
+CreateDesktopA.restype = HDESK
+DEVMODEA = c_void_p
+CreateDesktopA.argtypes = [LPCSTR, LPCSTR, DEVMODEA, DWORD, ACCESS_MASK, LPSECURITY_ATTRIBUTES]
+CreateDesktopExA = user32.CreateDesktopExA
+CreateDesktopExA.argtypes = [LPCSTR, LPCSTR, DEVMODEA, DWORD, ACCESS_MASK, LPSECURITY_ATTRIBUTES, ULONG, LPVOID]
+CreateDesktopExA = user32.CreateDesktopExA
+CreateDesktopExA.restype = HDESK
 OpenDesktopW = user32.OpenDesktopW
 OpenDesktopW.restype = HDESK
 OpenDesktopW.argtypes = [LPWSTR, DWORD, BOOL, ACCESS_MASK]
@@ -407,6 +411,10 @@ CloseDesktop.argtypes = [HDESK]
 OpenInputDesktop = user32.OpenInputDesktop
 OpenInputDesktop.restype = HDESK
 OpenInputDesktop.argtypes = [DWORD, BOOL, ACCESS_MASK]
+DESKTOPENUMPROCA = WINFUNCTYPE(BOOL, LPCSTR, LPARAM)
+EnumDesktopsA = user32.EnumDesktopsA
+EnumDesktopsA.restype = BOOL
+EnumDesktopsA.argtypes = [HWINSTA, DESKTOPENUMPROCA, LPARAM]
 GetUserObjectInformationA = user32.GetUserObjectInformationA
 GetUserObjectInformationA.restype = BOOL
 GetUserObjectInformationA.argtypes = [HANDLE, INT, LPVOID, DWORD, LPDWORD]
@@ -438,9 +446,6 @@ SetClipboardData.argtypes = [UINT, HANDLE]
 RegisterClipboardFormatA = user32.RegisterClipboardFormatA
 RegisterClipboardFormatA.restype = UINT
 RegisterClipboardFormatA.argtypes = [LPCSTR]
-GetClipboardFormatNameA = user32.GetClipboardFormatNameA
-GetClipboardFormatNameA.restype = c_int
-GetClipboardFormatNameA.argtypes = [UINT, LPCTSTR, c_int]
 RegisterClipboardFormatA = user32.RegisterClipboardFormatA
 RegisterClipboardFormatA.restype = UINT
 RegisterClipboardFormatA.argtypes = [LPCSTR]
@@ -483,6 +488,19 @@ EnumClipboardFormats.argtypes = [UINT]
 CountClipboardFormats = user32.CountClipboardFormats
 CountClipboardFormats.restype = c_int
 CountClipboardFormats.argtypes = []
+GetWindowTextA = user32.GetWindowTextA
+GetWindowTextA.restype = c_int
+GetWindowTextA.argtypes = [HWND, LPSTR, c_int]
+GetWindowTextW = user32.GetWindowTextW
+GetWindowTextW.restype = c_int
+GetWindowTextW.argtypes = [HWND, LPWSTR, c_int]
+GetWindowTextLengthA = user32.GetWindowTextLengthA
+GetWindowTextLengthA.restype = c_int
+GetWindowTextLengthA.argtypes = [HWND]
+GetWindowTextLengthW = user32.GetWindowTextLengthW
+GetWindowTextLengthW.restype = c_int
+GetWindowTextLengthW.argtypes = [HWND]
+
 
 class PROCESS_INFORMATION(Structure):
     _fields_ = (
