@@ -59,10 +59,6 @@ XINPUT_WHEEL_DIV = envint("XPRA_XINPUT_WHEEL_DIV", 15)
 DBUS_SCREENSAVER = envbool("XPRA_DBUS_SCREENSAVER", False)
 
 
-def get_xwindow(win):
-    from xpra.gtk_common.gtk_util import get_xwindow as _get_xwindow
-    return _get_xwindow(win)
-
 def gl_check():
     if not is_X11() and is_Wayland():
         return "disabled under wayland with GTK3 (buggy)"
@@ -454,7 +450,7 @@ def system_bell(window, device, percent, _pitch, _duration, bell_class, bell_id,
             #try to load it:
             from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings       #@UnresolvedImport
             device_bell = X11KeyboardBindings().device_bell
-        device_bell(get_xwindow(window), device, bell_class, bell_id, percent, bell_name)
+        device_bell(window.get_xid(), device, bell_class, bell_id, percent, bell_name)
     try:
         from xpra.gtk_common.error import xlog
         with xlog:
@@ -474,15 +470,13 @@ def _send_client_message(window, message_type, *values):
         X11Window = X11WindowBindings()
         root_xid = X11Window.getDefaultRootWindow()
         if window:
-            xid = get_xwindow(window)
+            xid = window.get_xid()
         else:
             xid = root_xid
         SubstructureNotifyMask = constants["SubstructureNotifyMask"]
         SubstructureRedirectMask = constants["SubstructureRedirectMask"]
         event_mask = SubstructureNotifyMask | SubstructureRedirectMask
-        from xpra.gtk_common.error import xsync
-        with xsync:
-            X11Window.sendClientMessage(root_xid, xid, False, event_mask, message_type, *values)
+        X11Window.sendClientMessage(root_xid, xid, False, event_mask, message_type, *values)
     except Exception as e:
         log.warn("failed to send client message '%s' with values=%s: %s", message_type, values, e)
 
@@ -543,13 +537,13 @@ def get_info():
     return i
 
 
-class XI2_Window(object):
+class XI2_Window:
     def __init__(self, window):
         log("XI2_Window(%s)", window)
         self.XI2 = X11XI2Bindings()
         self.X11Window = X11WindowBindings()
         self.window = window
-        self.xid = get_xwindow(window.get_window())
+        self.xid = window.get_window().get_xid()
         self.windows = ()
         self.motion_valuators = {}
         window.connect("configure-event", self.configured)
@@ -688,7 +682,7 @@ class XI2_Window(object):
         if dx!=0 or dy!=0:
             xinputlog("do_xi_motion(%s, %s) wheel deltas: dx=%i, dy=%i", event, device, dx, dy)
             #normalize (xinput is always using 15 degrees?)
-            client.wheel_event(wid, float(dx)/XINPUT_WHEEL_DIV, float(dy)/XINPUT_WHEEL_DIV, event.device)
+            client.wheel_event(wid, dx/XINPUT_WHEEL_DIV, dy/XINPUT_WHEEL_DIV, event.device)
 
     def get_pointer_extra_args(self, event):
         def intscaled(f):
@@ -709,7 +703,7 @@ class XI2_Window(object):
         return args
 
 
-class ClientExtras(object):
+class ClientExtras:
     def __init__(self, client, _opts):
         self.client = client
         self._xsettings_watcher = None
@@ -807,15 +801,6 @@ class ClientExtras(object):
             dbuslog.error(" %s", e)
             return
 
-        try:
-            import dbus
-            assert dbus
-        except ImportError as e:
-            dbuslog("setup_dbus_signals()", exc_info=True)
-            dbuslog.warn("Warning: cannot setup dbus signals")
-            dbuslog.warn(" %s", e)
-            return
-
         try:
             bus = init_system_bus()
             self.system_bus = bus
@@ -961,12 +946,10 @@ class ClientExtras(object):
 
 
     def _get_xsettings(self):
-        xw = self._xsettings_watcher
-        if xw:
-            try:
-                return self._xsettings_watcher.get_settings()
-            except:
-                log.error("failed to get XSETTINGS", exc_info=True)
+        try:
+            return self._xsettings_watcher.get_settings()
+        except:
+            log.error("failed to get XSETTINGS", exc_info=True)
         return None
 
     def _handle_xsettings_changed(self, *_args):
