@@ -6,6 +6,7 @@
 
 import os
 import signal
+from queue import Queue
 from multiprocessing import Process
 
 from xpra.server.proxy.proxy_instance import ProxyInstance
@@ -15,13 +16,13 @@ from xpra.net.protocol import Protocol
 from xpra.os_util import (
     SIGNAMES, POSIX,
     bytestostr,
-    Queue, osexpand,
+    osexpand,
     getuid, getgid, get_username_for_uid, setuidgid,
     register_SIGUSR_signals,
     )
 from xpra.util import (
     typedict,
-    repr_ellipsized,
+    ellipsizer,
     LOGIN_TIMEOUT, CONTROL_COMMAND_ERROR, AUTHENTICATION_ERROR, CLIENT_EXIT_TIMEOUT
     )
 from xpra.queue_scheduler import QueueScheduler
@@ -67,7 +68,7 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
                                video_encoder_modules, pings,
                                disp_desc, cipher, encryption_key, caps)
         QueueScheduler.__init__(self)
-        Process.__init__(self, name=str(client_conn))
+        Process.__init__(self, name=str(client_conn), daemon=False)
         self.client_conn = client_conn
         self.server_conn = server_conn
         self.uid = uid
@@ -77,9 +78,9 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
         self.client_state = client_state
         log("ProxyProcess%s", (uid, gid, env_options, session_options, socket_dir,
                                video_encoder_modules,
-                               client_conn, disp_desc, repr_ellipsized(str(client_state)),
+                               client_conn, disp_desc, ellipsizer(client_state),
                                cipher, encryption_key, server_conn,
-                               "%s: %s.." % (type(caps), repr_ellipsized(str(caps))), message_queue))
+                               "%s: %s.." % (type(caps), ellipsizer(caps)), message_queue))
         self.message_queue = message_queue
         #for handling the local unix domain socket:
         self.control_socket_cleanup = None
@@ -95,7 +96,7 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
 
 
     def server_message_queue(self):
-        while not self.exit and self.message_queue:
+        while not self.exit:
             log("waiting for server message on %s", self.message_queue)
             m = self.message_queue.get()
             log("received proxy server message: %s", m)
@@ -115,9 +116,17 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
     def signal_quit(self, signum, _frame=None):
         log.info("")
         log.info("proxy process pid %s got signal %s, exiting", os.getpid(), SIGNAMES.get(signum, signum))
+        QueueScheduler.stop(self)
+        self.message_queue.put_nowait(None)
         signal.signal(signal.SIGINT, deadly_signal)
         signal.signal(signal.SIGTERM, deadly_signal)
         self.stop(None, SIGNAMES.get(signum, signum))
+        #from now on, we can't rely on the main loop:
+        from xpra.os_util import register_SIGUSR_signals
+        register_SIGUSR_signals()
+        #log.info("instance frames:")
+        #from xpra.util import dump_all_frames
+        #dump_all_frames(log.info)
 
 
     ################################################################################
@@ -163,9 +172,9 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
         start_thread(self.server_message_queue, "server message queue")
 
         if not self.create_control_socket():
-            self.stop(None, "cannot create the proxy control socket")
+            #TODO: should send a message to the client
             return
-        self.control_socket_thread = start_thread(self.control_socket_loop, "control", daemon=True)
+        self.control_socket_thread = start_thread(self.control_socket_loop, "control")
 
         self.main_queue = Queue()
 
@@ -224,7 +233,7 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
         if cs:
             try:
                 self.control_socket.close()
-            except (OSError, IOError):
+            except OSError:
                 pass
         csc = self.control_socket_cleanup
         if csc:
@@ -234,15 +243,8 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
     def control_socket_loop(self):
         while not self.exit:
             log("waiting for connection on %s", self.control_socket_path)
-            try:
-                sock, address = self.control_socket.accept()
-            except OSError as e:
-                log("control_socket=%s", self.control_socket, exc_info=True)
-                log.error("Error accepting socket connection on %s", self.control_socket)
-                log.error(" %s", e)
-            else:
-                self.new_control_connection(sock, address)
-        self.control_socket_thread = None
+            sock, address = self.control_socket.accept()
+            self.new_control_connection(sock, address)
 
     def new_control_connection(self, sock, address):
         if len(self.potential_protocols)>=self.max_connections:
@@ -251,7 +253,7 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
             return  True
         try:
             peername = sock.getpeername()
-        except (OSError, IOError):
+        except OSError:
             peername = str(address)
         sockname = sock.getsockname()
         target = peername or sockname
@@ -322,7 +324,6 @@ class ProxyInstanceProcess(ProxyInstance, QueueScheduler, Process):
     ################################################################################
 
     def stop(self, skip_proto, *reasons):
-        QueueScheduler.stop(self)
-        self.message_queue.put_nowait(None)
-        ProxyInstance.stop(self, skip_proto, *reasons)
+        super().stop(skip_proto, *reasons)
         self.stop_control_socket()
+        QueueScheduler.stop(self)
