@@ -443,10 +443,14 @@ class XpraServer(GObject.GObject, X11ServerBase):
 
         root = get_default_root_window()
         with xsync:
-            for window in get_children(root):
-                xid = window.get_xid()
-                if X11Window.is_override_redirect(xid) and X11Window.is_mapped(xid):
-                    self._add_new_or_window(window)
+            children = get_children(root)
+        for window in children:
+            xid = window.get_xid()
+            can_add = False
+            with xlog:
+                can_add = X11Window.is_override_redirect(xid) and X11Window.is_mapped(xid)
+            if can_add:
+                self._add_new_or_window(window)
 
     def _lookup_window(self, wid):
         assert isinstance(wid, int), "window id value '%s' is a %s and not a number" % (wid, type(wid))
@@ -608,7 +612,8 @@ class XpraServer(GObject.GObject, X11ServerBase):
             return
         x, y, nw, nh = self._desktop_manager.window_geometry(window)
         resize_counter = self._desktop_manager.get_resize_counter(window, 1)
-        for ss in self._server_sources.values():
+        wsources = [ss for ss in self._server_sources.values() if isinstance(ss, WindowsMixin)]
+        for ss in wsources:
             ss.move_resize_window(wid, window, x, y, nw, nh, resize_counter)
             #refresh to ensure the client gets the new window contents:
             #TODO: to save bandwidth, we should compare the dimensions and skip the refresh
@@ -1025,7 +1030,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
                         #try to ensure this won't trigger a resizing loop:
                         counter = max(0, resize_counter-1)
                         for s in self._server_sources.values():
-                            if s!=ss:
+                            if s!=ss and isinstance(ss, WindowsMixin):
                                 s.resize_window(wid, window, aw, ah, resize_counter=counter)
                     damage |= owx!=ax or owy!=ay or resized
             if not shown and not skip_geometry:
